==> jekyll-embed-urls-0.3.1.gem: lib/jekyll-embed-urls.rb <==
require 'oembed'
require 'cgi'
require 'oga'

# TODO: We tested several of the mainstream embedable contents (YT, IG,
# Twitter) and specially IG and Twitter just want to take over the page
# to set their own size, also send metrics.  So they won't work on a
# sandboxed iframe, which we were expecting, but they also won't be
# comfortable for visitors to use.  We're planning on using OGP and
# render our own partials (configurable) for this.  This way everything
# is safer and the embedded content even adapts to the site's design.
#
# So, expect a major refactoring!

OEmbed::Providers.register_all
OEmbed::Providers.register_fallback(OEmbed::ProviderDiscovery,
                                    OEmbed::Providers::Noembed)

# Process the content of documents before rendering them to find URLs in
# a block.
Jekyll::Hooks.register :site, :pre_render do |site|
  # Cache results
  cache ||= Jekyll::Cache.new('Jekyll::OEmbed::Urls')
  # TODO: Make configurable
  referrer_policy = 'strict-origin-when-cross-origin'

  # Only modify documents to be written
  site.docs_to_write.each do |doc|
    # Skip text paragraphs
    # XXX: Find link in first line
    next unless %r{\n\n\s*<?https?://} =~ doc.content

    # Split texts by markdown blocks
    doc.content = doc.content.split("\n\n").map do |p|
      # Only process lines with URLs
      next p unless %r{\A\s*<?https?://} =~ p
      # Remove empty characters and markdown autolinks
      p = p.strip.tr('<', '').tr('>', '')

      # @see {https://developer.mozilla.org/en-US/docs/Web/HTML/Element/iframe#attr-sandbox}
      same_origin = p.start_with? site.config['url']

      Jekyll.logger.debug "Finding OEmbed content for #{p}"
      # Cache the results
      cache.getset(p) do
        Jekyll.logger.debug "=> Not cached, obtaining..."

        result = OEmbed::Providers.get(p)
        sandbox = "allow-scripts #{same_origin ? '' : 'allow-same-origin'}"

        # If the embed HTML contains an iframe, make sure it has the
        # correct attributes.
        if %r{<iframe } =~ result.html
          html = Oga.parse_html result.html

          html.css('iframe').each do |iframe|
            iframe.attributes.delete_if do |attr|
              %w[width height].include? attr.name
            end

            iframe.attributes << Oga::XML::Attribute.new(name: 'sandbox', value: sandbox)
            iframe.attributes << Oga::XML::Attribute.new(name: 'referrerpolicy', value: referrer_policy)
          end

          html.to_xml
        else
          # Return a sandboxed iframe with the size of the HTML.  We
          # only allow scripts to run inside the iframe and nothing
          # else.
          <<~IFRAME
            <iframe
              referrerpolicy="#{referrer_policy}"
              sandbox="#{sandbox}"
              style="min-width:#{result.width}px;min-height:#{result.height || 0}px"
              srcdoc="#{CGI.escape_html result.html}"></iframe>
          IFRAME
        end
      rescue OEmbed::NotFound => e
        # If the URL doesn't support OEmbed just return an external
        # link.
        #
        # TODO: Fetch information with OGP and render a template.
        Jekyll.logger.warn "#{p} is not oembeddable or URL can't be fetched, showing as URL"

        %(<p><a href="#{p}" target="_blank" referrerpolicy="#{referrer_policy}">#{p}</a></p>)
      end
    # Rebuild the content
    end.join("\n\n")
  end
end

==> jekyll-embed-urls-0.3.2.gem: lib/jekyll-embed-urls.rb <==
require 'oembed'
require 'cgi'
require 'oga'

# TODO: We tested several of the mainstream embedable contents (YT, IG,
# Twitter) and specially IG and Twitter just want to take over the page
# to set their own size, also send metrics.  So they won't work on a
# sandboxed iframe, which we were expecting, but they also won't be
# comfortable for visitors to use.  We're planning on using OGP and
# render our own partials (configurable) for this.  This way everything
# is safer and the embedded content even adapts to the site's design.
#
# So, expect a major refactoring!

OEmbed::Providers.register_all
OEmbed::Providers.register_fallback(OEmbed::ProviderDiscovery,
                                    OEmbed::Providers::Noembed)

# Process the content of documents before rendering them to find URLs in
# a block.
Jekyll::Hooks.register :site, :pre_render do |site|
  # Cache results
  cache ||= Jekyll::Cache.new('Jekyll::OEmbed::Urls')
  # TODO: Make configurable
  referrer_policy = 'strict-origin-when-cross-origin'

  # Only modify documents to be written
  site.docs_to_write.each do |doc|
    # Skip text paragraphs
    # XXX: Find link in first line
    next unless %r{\n\n\s*<?https?://} =~ doc.content

    # Split texts by markdown blocks
    doc.content = doc.content.split("\n\n").map do |p|
      # Only process lines with URLs
      next p unless %r{\A\s*<?https?://} =~ p
      # Remove empty characters and markdown autolinks
      p = p.strip.tr('<', '').tr('>', '')

      # @see {https://developer.mozilla.org/en-US/docs/Web/HTML/Element/iframe#attr-sandbox}
      same_origin = p.start_with? site.config['url']

      Jekyll.logger.debug "Finding OEmbed content for #{p}"
      # Cache the results
      cache.getset(p) do
        Jekyll.logger.debug "=> Not cached, obtaining..."

        result = OEmbed::Providers.get(p)
        sandbox = "allow-scripts #{same_origin ? '' : 'allow-same-origin'}"

        # If the embed HTML contains an iframe, make sure it has the
        # correct attributes.
        if %r{<iframe } =~ result.html
          html = Oga.parse_html result.html

          html.css('iframe').each do |iframe|
            iframe.attributes.delete_if do |attr|
              %w[width height].include? attr.name
            end

            iframe.attributes << Oga::XML::Attribute.new(name: 'sandbox', value: sandbox)
            iframe.attributes << Oga::XML::Attribute.new(name: 'referrerpolicy', value: referrer_policy)
          end

          html.to_xml
        else
          # Return a sandboxed iframe with the size of the HTML.  We
          # only allow scripts to run inside the iframe and nothing
          # else.
          <<~IFRAME
            <iframe
              referrerpolicy="#{referrer_policy}"
              sandbox="#{sandbox}"
              style="min-width:#{result.width}px;min-height:#{result.height || 0}px"
              srcdoc="#{CGI.escape_html result.html}"></iframe>
          IFRAME
        end
      rescue OEmbed::Error => e
        # If the URL doesn't support OEmbed just return an external
        # link.
        #
        # TODO: Fetch information with OGP and render a template.
        Jekyll.logger.warn "#{p} is not oembeddable or URL can't be fetched, showing as URL: #{e}"

        %(<p><a href="#{p}" target="_blank" referrerpolicy="#{referrer_policy}">#{p}</a></p>)
      end
    # Rebuild the content
    end.join("\n\n")
  end
end
